<?php
/**
 * DPSHostedPayment Class
 *
 * This module provides integration with DPS' Hosted Payment Page (via Px Pay)
 * https://www.paymentexpress.com/technical_resources/ecommerce_hosted/pxpay.html
 *
 * Requires:
 *		cURL (fairly common)
 *		SimpleXML (default in PHP 5.0+)
 */

 class DPSHostedPayment extends Payment {

	// The following settings can/should be set in yoursitename/_config.php
	// DPSHostedPayment::setConfig($setting, $value);
	protected static $pxpay_user_id;
	protected static $pxpay_key;
	protected static $url = 'https://sec2.paymentexpress.com/pxpay/pxaccess.aspx';
	protected static $logo = 'http://www.paymentexpress.com/images/dps_paymentexpress.png'; # dps|paymentexpress logo
	protected static $privacy = 'http://www.paymentexpress.com/privacypolicy.htm'; # link to dps' privacy policy
	protected static $txntype = 'Purchase'; # default transaction type if not specified in _config.php
	protected static $use_token_billing = false; # off by default (used with DPSTokenPayment.php)
	
	// Function to set relevant DPS Px Pay settings (done in youritename/_config.php)
	public static function setConfig($setting, $value) {
		self::$$setting = $value;
	}

	// This function is used by the DPSHostedPayment_Handler controller (below) to access static Px Pay settings
	public static function getConfig($setting) {
		return self::$$setting;
	}

	// This function is called to generate form fields when DPSHostedPayment is selected at checkout.
	// overrides same function in parent Payment class
	public function getPaymentFormFields() {

		$pxpay_fields = new FieldSet();
		// Horrible hack to insert space between payment method name and form fields
		$pxpay_fields->push(new LiteralField("space", "<p>&nbsp;</p>"));
		if (self::$use_token_billing) {
			$pxpay_fields->push(new CheckboxField("dps_save_card", 'Remember this card for next time'));
		}
		$pxpay_fields->push(new LiteralField("dps_redirect_notice", "<p>You will be redirected to a secure page to enter your credit card details.</p>"));
		$pxpay_fields->push(new LiteralField("dps_info", '<img src="' . self::$logo . '" alt="Payments powered by DPS" />'));

		return $pxpay_fields;
	}

	// Similar to getPaymentFormFields().  Provide checks on form fields with user-entered data.
	public function getPaymentFormRequirements() {
		return array();
	}

	// A helper function to handle background posts to DPS using cURL
	public static function doCurl($url, $content) {

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($ch, CURLOPT_POST, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $content);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

		$response = curl_exec($ch);
		if (curl_errno($ch) != 0) {
			curl_close($ch);
			return false;
		}
		else {
			curl_close ($ch);
			return $response;
		}
	}

	/**
	 * processPayment method
	 * Overrides processPayment() from parent Payment class
	 *
	 * Calculates 3rd-party payment page URL and redirects cardholder
	 * (this method usually processes payment directly when no 3rd party is involved)
	 */
	public function processPayment($data, $form) {

		$page = new Page();
		$page->Title = 'You are being Redirected to DPS | Payment Express';
		$page->Logo = '<img src="' . self::$logo . '" alt="Payments powered by DPS"/>';
		$page->Form = $this->buildDpsForm($data);

		$controller = new Page_Controller($page);

		Requirements::javascript(THIRDPARTY_DIR . '/jquery/jquery.js');

		$form = $controller->renderWith('PaymentProcessingPage');

		return new Payment_Processing($form);
	}

	/**
	 * buildDpsForm method
	 * This is called by processPayment method
	 *
	 * Submits transaction details to DPS and gets unique Hosted Payment Page URL
	 * Then builds and returns the redirect javascript with the unique URL
	 */
	public function buildDpsForm($data) {

		// Prepare values to populate XML fields to send to DPS
		$order_id = $this->OrderID;
		$currency = $this->Currency;
		$return_url = Director::absoluteBaseURL() . 'DPSHostedPayment_Handler/complete/';
		$amount_input = number_format($this->Amount, 2); # ensure the amount returns as d.cc format (5.00 as opposed to 5)

		// Build the XML to post to DPS
		$generateRequest = new SimpleXmlElement("<GenerateRequest></GenerateRequest>");
		$generateRequest->addChild("PxPayUserId", self::$pxpay_user_id);
		$generateRequest->addChild("PxPayKey", self::$pxpay_key);
		$generateRequest->addChild("AmountInput", $amount_input);
		$generateRequest->addChild("CurrencyInput", $currency);
		$generateRequest->addChild("TxnType", self::$txntype);
		$generateRequest->addChild("MerchantReference", "order #$order_id");
		$generateRequest->addChild("TxnData1", $order_id);
		$generateRequest->addChild("UrlSuccess", $return_url);
		$generateRequest->addChild("UrlFail", $return_url);
		// If token billing is enabled, ask DPS to save the card and return a card token
		if (self::$use_token_billing) {
			if (isset($data['dps_save_card'])) { # check for cardholder permission
				$generateRequest->addChild("EnableAddBillCard", '1');
			}
		}
		$xml = $generateRequest->asXML();

		// Post transaction XML to DPS using cURL.
		$xml = self::doCurl(self::$url, $xml); # cURL output is DPS' XML response containing payment page URL

		if ( ! $xml) {
			user_error('There was an error connecting to the payment gateway. ', E_USER_ERROR);
		}
		else {
			$request = simplexml_load_string($xml);
			$valid = $request['valid'];

			if ($valid) {
				$uri = $request->URI;
				$uri = Convert::raw2xml($uri);
				$snippet = '<script type="text/javascript">';
				$snippet .= 'jQuery(document).ready(function() {';
				$snippet .= "location = \"$uri\";";
				$snippet .= '});';
				$snippet .= '</script>';
				$snippet .= "<p>If you are not redirected automatically, please click <a href=\"$uri\">here</a> to proceed with payment.</p>";

				return $snippet;
			}
			else {
				user_error('There was a problem with the payment XML generated. ', E_USER_ERROR);
			}
		}
	}
}

/**
 * DPSHostedPayment_Handler Controller Class
 *
 * Responsible for determining transaction outcome when user returns from DPS payment page
 * Transaction outcome is encrypted in the result parameter passed in the URL - e.g:
 *
 * http://yoursite.com/DPSHostedPayment_Handler/complete?result=reallyLongEncryptedString
 *
 * The complete() method is called automatically because of URL routing
 */

class DPSHostedPayment_Handler extends Controller {

	// This method is called automatically because of URL routing
	function complete() {

		// DPS will return the transaction result encrypted in the URL
		if (isset($_REQUEST['result'])) {

			// build the xml for DPS to decrypt
			$processResponse = new SimpleXmlElement("<ProcessResponse></ProcessResponse>");
			$processResponse->addChild("PxPayKey", DPSHostedPayment::getConfig('pxpay_key'));
			$processResponse->addChild("PxPayUserId", DPSHostedPayment::getConfig('pxpay_user_id'));
			$processResponse->addChild("Response", $_REQUEST['result']);

			$xml = $processResponse->asXML();

			$xml = DPSHostedPayment::doCurl(DPSHostedPayment::getConfig('url'), $xml); # cURL output is DPS' XML response detailing transaction outcome

			if ( ! $xml) {
				user_error('There was an error connecting to the payment gateway.', E_USER_ERROR);
			}
			else {
				// Get some useful values from the transaction outcome xml
				$response = simplexml_load_string($xml);
				$success = $response->Success; # Success = 1, Failure = 0
				$txntype = $response->TxnType;
				$message = $response->ResponseText; # Message regarding the outcome of the transaction
				$ip = $response->ClientInfo; # the IP address used on the Hosted Payment Page
				$order_id = intval($response->TxnData1);
				$dpsbillingid = $response->DpsBillingId;
				// Now fetch the Payment entry from the database (marked as "Incomplete" until payment confirmed)
				$payment = DataObject::get_by_id('DPSHostedPayment', $order_id);
				
				if ($payment) {
					$payment->Status = ($success == '1') ? 'Success' : 'Failure';
					$payment->Message = sprintf($message);
					$payment->IP = sprintf($ip);
					$payment->write();

					/**
					 * Auth transactions need to be manually completed for funds to be transferred
					 *
					 * So when transaction type is "Auth", we mark the order as "Processing" in
					 * the CMS admin section.  Note that the order payment is marked as complete
					 * because the "Auth" transaction was successful.  You need to dive into the
					 * actual invoice to see the order marked as "Processing".
					 *
					 * This should be fine for low volume merchants that manually complete transactions
					 * after confirming having the requested stock.
					 */

					if ($success) {
						if ($txntype == "Auth") {
							$order = DataObject::get_by_id('Order', $order_id);
							$order->Status = "Processing";
							$order->write();
						}
					}

					/**
					 * If a DpsBillingId (token) is returned by DPS, then we need to store
					 * it for this user so it can be used in place of a credit card next time.
					 *
					 * A separate table in the database is setup for this: 'DPSToken'
					 * This requires DPSToken.php which sets up the database
					 */

					if ($dpsbillingid != '') {
						$DPSToken = new DPSToken();
						// Find this user's member id to associate the token with.
						$member = Member::currentUser();
						$member_id = $member->ID;
						$DPSToken->MemberID = sprintf($member_id);
						$DPSToken->TruncatedCard = sprintf($response->CardNumber);
						$DPSToken->Expiry = sprintf($response->DateExpiry);
						$DPSToken->DPSBillingId = sprintf($response->DpsBillingId);
						$DPSToken->write();
					}

					// Send the user to the order outcome screen
					$payment->redirectToOrder();
				}
				else {
					user_error("The transaction outcome information could not be decrypted successfully", E_USER_ERROR);
				}
			}
		}
		else {
			user_error('No transaction information to decrypt', E_USER_ERROR);
		}
	}
}
?>