<?php
/**
 * DPSTokenPayment Class
 *
 * This module complements DPSHostedPayment by facilitating easy repeat purchases
 * Relies on DPSHostedPayment to generate a token during first transaction
 * Stores a token in the database and uses this to process subsequent transactions
 *
 * Requires:
 *		DPSToken.php (stores token info in database)
 *		Token billing to be enabled for DPSHostedPayment.php
 *		SimpleXML (default in PHP 5.0+)
 *		SOAP
 */

/**
 * TODO: Consider rewrite for Px Post (same credentials as Web Service)
 *
 * SOAP web service supports a method to update expiry date of saved cards
 * ...but Px Post doesn't require SOAP, only cURL which is more common.
 * ...maybe DPS can add functionality to update expiry to Px Post
 */

class DPSTokenPayment extends Payment {

	// The following settings can/should be set in yoursitename/_config.php
	// DPSTokenPayment::setConfig($setting, $value);
	protected static $pxpost_username;
	protected static $pxpost_password;
	protected static $wsdl = 'https://www.paymentexpress.com/WS/PXWS.asmx?WSDL';
	protected static $logo = 'http://www.paymentexpress.com/images/dps_paymentexpress.png'; # dps|paymentexpress logo

	// Function to set relevant DPS Web Service settings (done in youritename/_config.php)
	public static function setConfig($setting, $value) {
		self::$$setting = $value;
	}

	// This function is called to generate form fields when DPSHostedPayment is selected at checkout.
	// overrides same function in parent Payment class
	public function getPaymentFormFields() {

		$dps_tokens = array(); # this will store any tokens belonging to current member from database

		// Check if the user has any saved credit cards
		$member = Member::currentUser();
		if ($member) {
			$member_id = $member->ID;
			$member_id = Convert::raw2sql($member_id);
			$stored_card_details = DataObject::get("DPSToken", "DPSToken.MemberID = '$member_id'");

			// Get some info about the stored cards to display to the user
			if ($stored_card_details) {
				$i = 0;
				foreach ($stored_card_details as $object) {
					$dps_tokens[$i]['TruncatedCard'] = $object->TruncatedCard;
					$dps_tokens[$i]['ID'] = $object->ID;
					$dps_tokens[$i]['Expiry'] = $object->Expiry;
					$i++;
				}
				unset($i);
			}
		}

		$pxws_fields = new FieldSet();
		// Horrible hack to insert space between payment method name and form fields
		$pxws_fields->push(new LiteralField("space", "<p>&nbsp;</p>"));
		if (count($dps_tokens) > 0) {
			foreach ($dps_tokens as $token) {
				// Create a radio button set for choosing stored card
				$token_id = $token['ID'];
				$radio_set[$token_id] = $token['TruncatedCard'] . " (exp: " . $token['Expiry'] . ")";
			}
			$pxws_fields->push(new OptionsetField("pxws_token_id", 'Card: ', $radio_set, "1"));
			$pxws_fields->push(new TextField('pxws_expiry', 'Update selected card\'s expiry (mmyy)', '', 4));
		}
		else {
			$pxws_fields->push(new LiteralField("no_cards", '<p><b>No saved cards found.</b> Please choose a different payment method.</p>'));
		}
		$pxws_fields->push(new LiteralField("DPSLogo", '<img src="' . self::$logo . '" alt="Payments powered by DPS" />'));

		return $pxws_fields;
	}

	// Similar to getPaymentFormFields().  Provide checks on form fields with user-entered data.
	// Should use this to validate expiry date if user enters one but not sure how yet...
	public function getPaymentFormRequirements() {
		return array();
	}

	// This function overrides processPayment() from parent Payment class
	// Responsible for processing the payment
	function processPayment($data, $form) {

		// Collect other necessary transaction information
		$order_id = $this->OrderID;
		$currency = $this->Currency;
		$amount = number_format($this->Amount, 2); # ensure amount is d.cc format (5.00 as opposed to 5)

		// Collect data required to fetch the nominated token from database
		$token_id = Convert::raw2sql($data['pxws_token_id']);
		$member = Member::currentUser();
		$member_id = $member->ID; # used to verify that the token belongs to current user

		// Fetch the token from the database
		$token = DataObject::get_one('DPSToken', "DPSToken.ID = '$token_id' AND DPSToken.MemberID = '$member_id'");
		if ($token) {
			$dpsbillingid = $token->DPSBillingId;
		}
		else {
			user_error("No such token found", E_USER_ERROR);
		}

		// The user is given the option to update the expiry date of their nominated card at checkout
		// So we need to update this at DPS before actual transaction takes place
		$expiry = isset($data['pxws_expiry']) ? trim($data['pxws_expiry']) : false;
		if ($expiry) {
			if (strlen($expiry) == 4) {
				if (self::updateCardExpiry($dpsbillingid, $expiry)) {
					// Update the card's expiry in database
					$token->Expiry = Convert::raw2sql($expiry);
					$token->write();
					// if...else will exit and payment will continue as normal
				}
				else {
					$this->Message = "Updating card expiry failed";
					$this->Status = "Failure";
					$this->write();
					return new Payment_Failure($amount);
				}
			}
			else {
				// Assume Payment will fail because invalid expiry was supplied (not 4 characters)
				$this->Message = "Invalid card expiry supplied";
				$this->Status = "Failure";
				$this->write();
				return new Payment_Failure($amount);
			}
		}

		// Build the data array to pass onto soap client
		$soap_array = array(
			'postUsername' => self::$pxpost_username,
			'postPassword' => self::$pxpost_password,
			'transactionDetails' => array(
				'txnType' => DPSHostedPayment::getConfig('txntype'),
				'amount' => $amount,
				'dpsBillingId' => $dpsbillingid,
				'inputCurrency' => $currency,
				'txnData1' => $order_id,
				'merchantReference' => "order #$order_id"
			)
		);

		// Do the transaction using PHP SOAP client
		$client = new SoapClient(self::$wsdl, array('soap_version' => SOAP_1_2));
		$response = $client->SubmitTransaction($soap_array);

		// Gather some info from transaction outcome
		$success = $response->SubmitTransactionResult->authorized;
		$txntype = $response->SubmitTransactionResult->txnType;
		$message = $response->SubmitTransactionResult->responseText;
		$unresolved = $response->SubmitTransactionResult->statusRequired; # transaction outcome unknown

		// First check if a definitive transaction outcome was reached.
		if ($unresolved) # connection between DPS and banks may have failed during transaction.
		{
			// Log payment as pending
			$this->Status = "Pending";
			$this->Message = "Transaction outcome not yet known";
			$this->write();
			// Mark the order as requiring manual investigation
			$order = DataObject::get_by_id('Order', $order_id);
			$order->Status = "Query";
			$order->write();

			return new Payment_Processing($amount); # payment is in limbo
		}
		elseif ($success)
		{
			// Transaction was successful
			$this->Status = "Success";
			$this->Message = sprintf(Convert::raw2sql($message));
			$this->write();
			// If the transaction was an Auth, mark the order as requiring manual completion
			if ($txntype == "Auth") {
					$order = DataObject::get_by_id('Order', $order_id);
					$order->Status = "Processing";
					$order->write();
			}
			return new Payment_Success($amount);
		}
		else
		{
			// Transaction was declined or otherwise failed
			$this->Status = "Failure";
			$this->Message = sprintf(Convert::raw2sql($message));
			$this->write();

			return new Payment_Failure($amount);
		}
	}

	// Function to update the expiry dates of cards stored in tokens
	public static function updateCardExpiry($token, $expiry) {
		// create <cardDetails> XML to send to DPS via SOAP web service
		// http://www.paymentexpress.com/technical_resources/ecommerce_nonhosted/webservice.html#UpdateCard
		$card_details = "<cardDetails><dpsBillingId>$token</dpsBillingId><dateExpiry>$expiry</dateExpiry></cardDetails>";
		$array = array(
				'postUsername' => self::$pxpost_username,
				'postPassword' => self::$pxpost_password,
				'cardDetails' => $card_details
		);

		$client = new SoapClient("https://www.paymentexpress.com/WS/PXWS.asmx?WSDL", array('soap_version' => SOAP_1_2));
		$response = $client->UpdateCard($array);
		$updateCardResult = $response->UpdateCardResult;

		// Find the <updateSucceeded> field in the response XML
		$xml = simplexml_load_string($updateCardResult);
		$success = $xml->updateSucceeded;

	return ($success) ? true : false;
	}
}
?>
